"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const listr_1 = tslib_1.__importDefault(require("listr"));
const promise_retry_1 = tslib_1.__importDefault(require("promise-retry"));
const errors_1 = tslib_1.__importDefault(require("@tradle/errors"));
const errors_2 = require("./errors");
const logger_1 = require("./logger");
const USE_CURRENT_USER_ROLE = true;
exports.update = (conf, { stackId, tag, provider, showReleaseCandidates, force, }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const getUpdateWithRetry = tag => {
        let requested;
        return promise_retry_1.default((retry, number) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                return yield conf.getUpdateInfo({ tag });
            }
            catch (err) {
                errors_1.default.rethrow(err, ['developer', errors_2.Errors.InvalidInput]);
                if (!requested && errors_1.default.matches(err, errors_2.Errors.NotFound)) {
                    requested = true;
                    yield conf.requestUpdate({ tag, provider });
                }
                retry(err);
            }
        }), {
            maxTimeout: 10000,
            minTimeout: 5000,
            retries: 10
        });
    };
    if (!tag) {
        let updates = yield conf.listUpdates({ provider });
        if (!showReleaseCandidates) {
            updates = updates.filter(update => !isReleaseCandidateTag(update.tag));
        }
        if (!updates.length) {
            logger_1.logger.info(`no updates available`);
            return;
        }
        if (updates.length === 1) {
            tag = updates[0].tag;
            const { doUpdate } = yield inquirer_1.default.prompt([{
                    type: 'confirm',
                    name: 'doUpdate',
                    message: `Update to version ${tag} ?`,
                }]);
            if (!doUpdate)
                return;
        }
        else {
            const result = yield inquirer_1.default.prompt([{
                    type: 'list',
                    name: 'tag',
                    message: 'Choose a version to update to',
                    choices: updates.map(u => u.tag),
                }]);
            tag = result.tag;
        }
    }
    const triggerUpdate = (update) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (USE_CURRENT_USER_ROLE) {
            yield conf.applyUpdateAsCurrentUser(update);
        }
        else {
            yield conf.applyUpdateViaLambda(update);
        }
    });
    const promise = yield new listr_1.default([
        {
            title: 'download update (grab a coffee)',
            task: (ctx) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const resp = yield getUpdateWithRetry(tag);
                if (!resp)
                    return;
                const { update, upToDate } = resp;
                ctx.upToDate = upToDate;
                ctx.update = update;
                ctx.willUpdate = force || !upToDate;
            })
        },
        {
            title: 'validate update',
            skip: ctx => !ctx.willUpdate,
            task: (ctx) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield triggerUpdate(ctx.update);
            })
        },
        {
            title: 'apply update (be patient, or else)',
            skip: ctx => !ctx.willUpdate,
            task: (ctx) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield conf.waitForStackUpdate(stackId);
            })
        }
    ]).run();
    let ctx;
    try {
        ctx = yield promise;
    }
    catch (err) {
        if (errors_1.default.matches(err, errors_2.Errors.NotFound)) {
            logger_1.logger.error('failed to fetch the requested update');
        }
        throw err;
    }
    if (ctx.upToDate && !ctx.willUpdate) {
        logger_1.logger.info(`your MyCloud is already up to date!`);
    }
});
const isReleaseCandidateTag = (tag) => /-rc\.\d+$/.test(tag);
//# sourceMappingURL=update.js.map